%----------------------------------------------------------------------
%the sparse coding problem with two l1 regularization entries, one
%is the l1-norm of coding coef, the other is the l1-norm of coding
%coef minus the mean
%the object funciton is
% arg min{||y-Ax||_F^2+tau1*||x||_1+tau2*||x-m||_1}
% using surrogate function with iterative algorithm

%           Inputs
%                 ipts           -         an input data structure
%                     .D         -         the A matrix, each column vector
%                                          of A denotes a training sample
%                     .tau1      -         the parameter tau1
%                     .tau2      -         the paremeter tau2
%                     .m         -         the mean of coeficient
%                     .y         -         an input sample column vector
%                     .initM     -         initial method
%                     .twist     -         true: twist acceleration;false:no
%                                          accleration
%                     .citeT    -         the relative citeration threhold,
%                                          default:1e-4
%                     .cT        -         the stop citeration of c, the
%                                          threhold
%
%                 par            -         an input parameter structure
%                     .nIter     -         the total iteration number
%                     .cRatio    -         the times of c to max eigenvalue
% 
%            Outputs
%                opts            -         an output datastructure
%                     .x         -         the output spase coef
%                     .ert       -         the total error in iteration
%----------------------------------------------------------------------

function [opts] = shrinkage_l1 (dict,lambda,y)

% tic;

x(:,1)      =         zeros(size(dict,1)*size(dict,2),1);
f_m         =         size(dict,1);
A           =         dict;    % the diagonal elements
tau1        =         lambda;
twist       =         true;
nIter       =         200;
cRatio      =         1.05;
c           =         cRatio*max(max(A.*A));

citeT              =         1e-6;
cT                 =         1e+10;
for_ever           =         1;
IST_iters          =         0;
TwIST_iters        =         0;
sparse_sign        =         1;
verbose            =         1;
enforceMonotone    =         1;
lam1               =         1e-4;   %default eigenvalues
lamN               =         1;      %default eigenvalues
rho0               =         (1-lam1/lamN)/(1+lam1/lamN); 
alpha              =         2/(1+sqrt(1-rho0^2));        %default,user can set
beta               =         alpha*2/(lam1+lamN);         %default,user can set

%-------------------------------------------------------------
%start of main loop
%-------------------------------------------------------------
if ~twist
for n_it = 2 : nIter
    A_x          =     A(:,1).*x(1:f_m,n_it-1)+A(:,2).*x(f_m+1:end,n_it-1);
    ert(n_it-1)  =     norm(y-A_x,2)^2 + tau1*sum(abs(x(:,n_it-1)));
    y_A_x        =     y - A_x;
    v            =     1/c*[A(:,1).*y_A_x;A(:,2).*y_A_x]+x(:,n_it-1);
    x(:,n_it)    =     soft (v, tau1/c/2);    
end

else
    
   xm2       =      x(:,1);
   xm1       =      x(:,1);
   A_x       =      A(:,1).*x(1:f_m,1)+A(:,2).*x(f_m+1:end,1);
   prev_f    =      norm(y-A_x,2)^2+tau1*sum(abs(x(:,1)));  
   
   for n_it = 2 : nIter
       
   A_x              =     A(:,1).*x(1:f_m,n_it-1)+A(:,2).*x(f_m+1:end,n_it-1);
   ert(n_it-1)      =     norm(y-A_x,2)^2+tau1*sum(abs(x(:,n_it-1)));

    while for_ever
        % IST estimate  
        A_x          =     A(:,1).*xm1(1:f_m)+A(:,2).*xm1(f_m+1:end);
        y_A_x        =     y - A_x;
        v            =     1/c*[A(:,1).*y_A_x;A(:,2).*y_A_x]+xm1;
        x_temp       =     soft ( v, tau1/c/2);
    
       if (IST_iters >= 2) | ( TwIST_iters ~= 0)
            % set to zero the past when the present is zero
            % suitable for sparse inducing priors
            if sparse_sign
                mask    =   (x_temp ~= 0);
                xm1     =   xm1.* mask;
                xm2     =   xm2.* mask;
            end
            % two-step iteration
            xm2    =   (alpha-beta)*xm1 + (1-alpha)*xm2 + beta*x_temp;
            % compute residual
            A_x    =     A(:,1).*xm2(1:f_m)+A(:,2).*xm2(f_m+1:end);
            f      =   norm(y-A_x,2)^2+ tau1*sum(abs(xm2));
            if (f > prev_f) & (enforceMonotone)
                TwIST_iters   =  0;  % do a IST iteration if monotonocity fails
            else
                TwIST_iters =   TwIST_iters+1; % TwIST iterations
                IST_iters   =    0;
                x_temp      =   xm2;
                if mod(TwIST_iters,10000) ==0
                   c = 0.9*c; 
                end
                break;  % break loop while
            end
       else
            A_x         =     A(:,1).*x_temp(1:f_m)+A(:,2).*x_temp(f_m+1:end);
            f           =     norm(y-A_x,2)^2+tau1*sum(abs(x_temp));
            
            if f > prev_f
                % if monotonicity  fails here  is  because
                % max eig (A'A) > 1. Thus, we increase our guess
                % of max_svs
                c         =    2*c;        
                if verbose
%                     fprintf('Incrementing c=%2.2e\n',c);
                end
                if  c > cT
                    break;  % break loop while    
                end
                IST_iters = 0;
                TwIST_iters = 0;
            else
                TwIST_iters = TwIST_iters + 1;
                break;  % break loop while
            end
        end
    end

    citerion      =   abs(f-prev_f)/prev_f;
    if citerion < citeT | c > cT
%        fprintf('Stop!\n c=%2.2e\n citerion=%2.2e\n',c,citerion);
       break;
    end
    
    xm2           =   xm1;
    xm1           =   x_temp;
    x(:,n_it)     =   x_temp;
    prev_f        =   f;
    
   end

end
%--------------------------------------------------------------------------
% end of main loop
%--------------------------------------------------------------------------
opts.x     =       x(:,end);
opts.ert   =       ert;
% toc
% close all;plot(ert);pause(3);